package de.intevation.flys.artifacts.model.sq;

import java.util.List;

import de.intevation.flys.artifacts.model.DateRange;

import de.intevation.flys.backend.SedDBSessionHolder;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import org.hibernate.transform.BasicTransformerAdapter;

import org.hibernate.type.StandardBasicTypes;

public class MeasurementFactory
{
    private static final Logger log =
        Logger.getLogger(MeasurementFactory.class);

    public static final String SQL_MEASSURE =
        "SELECT m.datum        AS DATUM," +
               "g.UFERABST     AS UFERABST," +
               "g.UFERABLINKS  AS UFERABLINKS," +
               "m.tschweb      AS TSCHWEB," +
               "m.TSAND        AS TSAND," +
               "gp.MESSDAUER   AS MESSDAUER," +
               "gp.MENGE       AS MENGE," +
               "gp.GTRIEB      AS GTRIEB," +
               "gp.LFDNR       AS LFDNR," +
               "m.TGESCHIEBE   AS TGESCHIEBE," +
               "gs.RSIEB01     AS RSIEB01," +
               "gs.RSIEB02     AS RSIEB02," +
               "gs.RSIEB03     AS RSIEB03," +
               "gs.RSIEB04     AS RSIEB04," +
               "gs.RSIEB05     AS RSIEB05," +
               "gs.RSIEB06     AS RSIEB06," +
               "gs.RSIEB07     AS RSIEB07," +
               "gs.RSIEB08     AS RSIEB08," +
               "gs.RSIEB09     AS RSIEB09," +
               "gs.RSIEB10     AS RSIEB10," +
               "gs.RSIEB11     AS RSIEB11," +
               "gs.RSIEB12     AS RSIEB12," +
               "gs.RSIEB13     AS RSIEB13," +
               "gs.RSIEB14     AS RSIEB14," +
               "gs.RSIEB15     AS RSIEB15," +
               "gs.RSIEB16     AS RSIEB16," +
               "gs.RSIEB17     AS RSIEB17," +
               "gs.RSIEB18     AS RSIEB18," +
               "gs.RSIEB19     AS RSIEB19," +
               "gs.RSIEB20     AS RSIEB20," +
               "gs.RSIEB21     AS RSIEB21," +
               "gs.REST        AS REST," +
               "g.GLOTRECHTEID AS GLOTRECHTEID " +
        "FROM MESSUNG m " +
            "JOIN STATION    s ON m.STATIONID    = s.STATIONID " +
            "JOIN glotrechte g ON m.MESSUNGID    = g.MESSUNGID " +
            "JOIN gprobe    gp ON g.GLOTRECHTEID = gp.GLOTRECHTEID " +
            "JOIN GSIEBUNG  gs ON g.GLOTRECHTEID = gs.GLOTRECHTEID " +
        "WHERE " +
            "g.NAME = :river_name " +
            "AND m.Q_BPEGEL IS NOT NULL " +
            "AND s.KM BETWEEN :location - 0.001 AND :location + 0.001 " +
            "AND m.DATUM BETWEEN :from AND :to " +
            "AND m.TGESCHIEBE IS NOT NULL " +
        "ORDER BY m.DATUM";

    public static final class MeasurementResultTransformer
    extends                   BasicTransformerAdapter
    {
        public static MeasurementResultTransformer INSTANCE =
            new MeasurementResultTransformer();

        public MeasurementResultTransformer() {
        }

        @Override
        public Object transformTuple(Object [] tuple, String [] aliases) {
            Map<String, Object> map = new HashMap<String, Object>();
            for (int i = 0; i < tuple.length; ++i) {
                if (tuple[i] != null) {
                    map.put(aliases[i], tuple[i]);
                }
            }
            return new Measurement(map);
        }
    } // class BasicTransformerAdapter

    private MeasurementFactory() {
    }

    protected static Measurements load(
        Session   session,
        String    river,
        double    location,
        DateRange dateRange
    ) {
        boolean debug = log.isDebugEnabled();

        if (debug) {
            log.debug(SQL_MEASSURE);
        }

        SQLQuery query = session.createSQLQuery(SQL_MEASSURE)
            .addScalar("TSAND",        StandardBasicTypes.DOUBLE)
            .addScalar("TSCHWEB",      StandardBasicTypes.DOUBLE)
            .addScalar("CSCHWEB",      StandardBasicTypes.DOUBLE)
            .addScalar("Q_BPEGEL",     StandardBasicTypes.DOUBLE)
            .addScalar("DATUM",        StandardBasicTypes.DATE)
            .addScalar("UFERABST",     StandardBasicTypes.DOUBLE)
            .addScalar("UFERABLINKS",  StandardBasicTypes.DOUBLE)
            .addScalar("TSCHWEB",      StandardBasicTypes.DOUBLE)
            .addScalar("TSAND",        StandardBasicTypes.DOUBLE)
            .addScalar("MESSDAUER",    StandardBasicTypes.DOUBLE)
            .addScalar("MENGE",        StandardBasicTypes.DOUBLE)
            .addScalar("GTRIEB",       StandardBasicTypes.DOUBLE)
            .addScalar("LFDNR",        StandardBasicTypes.DOUBLE)
            .addScalar("TGESCHIEBE",   StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB01",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB02",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB03",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB04",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB05",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB06",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB07",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB08",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB09",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB10",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB11",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB12",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB13",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB14",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB15",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB16",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB17",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB18",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB19",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB20",      StandardBasicTypes.DOUBLE)
            .addScalar("RSIEB21",      StandardBasicTypes.DOUBLE)
            .addScalar("REST",         StandardBasicTypes.DOUBLE)
            .addScalar("GLOTRECHTEID", StandardBasicTypes.DOUBLE);

        query.setString("river_name", river);
        query.setDouble("location", location);
        query.setDate("from", dateRange.getFrom());
        query.setDate("to", dateRange.getTo());

        query.setResultTransformer(MeasurementResultTransformer.INSTANCE);

        @SuppressWarnings("unchecked")
		List<Measurement> measuments = (List<Measurement>)query.list();

        for (int i = 0, N = measuments.size(); i < N; ++i) {
            Measurement m = measuments.get(i);
            Measurement p = i >   0 ? measuments.get(i-1) : null;
            Measurement n = i < N-1 ? measuments.get(i+1) : null;
            m.setPrev(p);
            m.setNext(n);
        }

        return new Measurements(measuments);
    }

    public static Measurements getMeasurements(
        String    river,
        double    location,
        DateRange dateRange
    ) {
        Session session = SedDBSessionHolder.HOLDER.get();
        try {
            return load(session, river, location, dateRange);
        }
        finally {
            session.close();
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
